@@ -1,5 +1,10 @@
 '''
-Defines the user interface for gaim.
+Defines corrections for the metacity desktop and app switcher.
+
+Event announcements
+
+on cycle windows: [name of window]
+on cycle desktops: [name of desktop]
 
 @author: Brett Clippingdale
 @author: Peter Parente
@@ -13,10 +18,14 @@ this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
 '''
 import Perk, Task
+import LSRConstants
+
+__uie__ = dict(kind='perk', tier='metacity')
 
 class MetacityPerk(Perk.Perk):
   '''
-  Creates and registers L{Task}s to improve the user interface for Metacity.
+  Announce information in the metacity desktop and application switcher 
+  windows. Supplements the DefaultPerk.
   '''
   def init(self):
     '''   
@@ -27,17 +36,16 @@ class MetacityPerk(Perk.Perk):
     # register event handlers
     self.registerEventTask(HandlePropertyChange(self), tier=True)
 
-    
 class HandlePropertyChange(Task.PropertyTask):
   '''  
-  Task that handles a L{AEEvent.PropertyChange}.
+  Announces Alt-Tab application switching by reading the accessible name of
+  the status text when it changes.
   '''  
   def execute(self, por, name, value, **kwargs):
-    '''
-    Announces alt-tab application switching.
-    '''
-    text = self.getItemText(por)  # get the selected text
-    self.stopNow()
-    self.say(text)
+    # get the text in status bar
+    text = self.getItemText(por)
+    # pretend we're on the focus layer even though text is changing elsewhere
+    self.stopNow(layer=Task.LAYER_FOCUS)
+    self.say(text=text, sem=LSRConstants.SEM_WINDOW, layer=Task.LAYER_FOCUS)
     return True
 
